//----------------------------------------------------------------------------
//-- Fractional Multiplier for Mandelbrot Machine
//----------------------------------------------------------------------------
module fmul(
	input	[15:0]md,	// Multiplicand
	input	[15:0]mr,	// Multiplier
	output 	[15:0]out	// Product
);

wire [15:0]d;		// Multiplicand (sign adjusted)
wire [14:0]r;		// Multiplier (positive)
wire [17:0]sum;		// result with 2 extra bits

// Make multiplier positive
assign r=mr[15]? ((~mr[14:0])+1) : mr[14:0];
assign d=mr[15]? ((~md)+1) : md;

wire s;	// sign bit of multiplicand
assign s=d[15];

assign sum= 18'h3 +		// (= 0.75 LSB) for rounding instead of truncating
 (r[14]? {d[12:0],5'h0}: 18'h0) +
 (r[13]? {d[13:0],4'h0}: 18'h0) +
 (r[12]? {d[14:0],3'h0}: 18'h0) +
 (r[11]? {d[15:0],2'h0}: 18'h0) +
 (r[10]? {s,d[15:0],1'h0}: 18'h0) +
 (r[9]?  {s,s,d[15:0]}: 18'h0) +
 (r[8]?  {s,s,s,d[15:1]}: 18'h0) +
 (r[7]?  {s,s,s,s,d[15:2]}: 18'h0) +
 (r[6]?  {s,s,s,s,s,d[15:3]}: 18'h0) +
 (r[5]?  {s,s,s,s,s,s,d[15:4]}: 18'h0) +
 (r[4]?  {s,s,s,s,s,s,s,d[15:5]}: 18'h0) +
 (r[3]?  {s,s,s,s,s,s,s,s,d[15:6]}: 18'h0) +
 (r[2]?  {s,s,s,s,s,s,s,s,s,d[15:7]}: 18'h0) +
 (r[1]?  {s,s,s,s,s,s,s,s,s,s,d[15:8]}: 18'h0) +
 (r[0]?  {s,s,s,s,s,s,s,s,s,s,s,d[15:9]}: 18'h0) ;
	
assign out=sum[17:2];

endmodule

//----------------------------------------------------------------------------
//-- Fractional Multiplier for Mandelbrot Machine
//----------------------------------------------------------------------------
module fmul24(
	input	[23:0]md,	// Multiplicand
	input	[23:0]mr,	// Multiplier
	output 	[23:0]out	// Product
);

wire [23:0]d;		// Multiplicand (sign adjusted)
wire [22:0]r;		// Multiplier (positive)
wire [25:0]sum;		// result with 2 extra bits

// Make multiplier positive
assign r=mr[23]? ((~mr[22:0])+1) : mr[22:0];
assign d=mr[23]? ((~md)+1) : md;

wire s;	// sign bit of multiplicand
assign s=d[23];

assign sum= 26'h3 +		// (= 0.75 LSB) for rounding instead of truncating
 (r[22]? {d[20:0],5'h0}: 26'h0) +
 (r[21]? {d[21:0],4'h0}: 26'h0) +
 (r[20]? {d[22:0],3'h0}: 26'h0) +
 (r[19]? {d[23:0],2'h0}: 26'h0) +
 (r[18]? {s,d[23:0],1'h0}: 26'h0) +
 (r[17]? {s,s,d[23:0]}: 26'h0) +
 (r[16]? {s,s,s,d[23:1]}: 26'h0) +
 (r[15]? {s,s,s,s,d[23:2]}: 26'h0) +
 (r[14]? {s,s,s,s,s,d[23:3]}: 26'h0) +
 (r[13]? {s,s,s,s,s,s,d[23:4]}: 26'h0) +
 (r[12]? {s,s,s,s,s,s,s,d[23:5]}: 26'h0) +
 (r[11]? {s,s,s,s,s,s,s,s,d[23:6]}: 26'h0) +
 (r[10]? {s,s,s,s,s,s,s,s,s,d[23:7]}: 26'h0) +
 (r[9]?  {s,s,s,s,s,s,s,s,s,s,d[23:8]}: 26'h0) +
 (r[8]?  {s,s,s,s,s,s,s,s,s,s,s,d[23:9]}: 26'h0) +
 (r[7]?  {s,s,s,s,s,s,s,s,s,s,s,s,d[23:10]}: 26'h0) +
 (r[6]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:11]}: 26'h0) +
 (r[5]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:12]}: 26'h0) +
 (r[4]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:13]}: 26'h0) +
 (r[3]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:14]}: 26'h0) +
 (r[2]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:15]}: 26'h0) +
 (r[1]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:16]}: 26'h0) +
 (r[0]?  {s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,d[23:17]}: 26'h0) ;
	
assign out=sum[25:2];

endmodule
