////////////////////////////////////////////////////////////////
//						VIDEO generation
////////////////////////////////////////////////////////////////

module videomodule ( 
	input clk,		// clock input @25MHz
	input [15:0]d,	// data read from external RAM
	output [15:0]a,	// external RAM address
	output hsyn,	// Horizontal sync. pulse (active low)
	output vsyn,	// Vertical sync. pulse (active low)
	output [11:0]video,	// Video output (12 bits/pixel)
	output vrd,		// video read
	output hblk,	// Horizontal blank
	output vblk,	// Vertical blank
	output [9:0]hc	// Horizontal counter
);

// Horizontal section
// Total: 800 pixels (32 us)
// Displayed: 512 pixels
// hsync width: 96 pixels
reg [9:0]hc=0;	// Horizontal counter
reg hsyn=1;		// sync
reg hblk=0;		// blank output (active if pixel >= 512)
always @(posedge clk) begin
	if (hc==10'd799) begin hc<=0; hblk<=0; end else hc<=hc+1;
	if (hc==10'd591) hsyn<=0;
	if (hc==10'd687) hsyn<=1;
	if (hc==10'd511) hblk<=1;
end

// Vertical section
// Total: 525 lines
// Displayed: 480 lines
// vsync width: 2 lines
reg [9:0]vc=0;	// Vertical Counter
reg vsyn=1;		// sync.
reg vblk=0;		// blank output (active if line >=480)
always @(negedge hblk) begin
	if (vc==10'd524) begin vc<=0; vblk<=0; end else vc<=vc+1;
	if (vc==10'd489) vsyn<=0;
	if (vc==10'd491) vsyn<=1;
	if (vc==10'd479) vblk<=1;
end

// Global Blank
wire blk;
assign blk = hblk|vblk;

// Memory Address
assign a = {vc[8:0],hc[8:2]};
reg dblk=0;	// blk delayed
always @(posedge clk) dblk<=blk;

// Video read
assign vrd = (~blk) & (~hc[1]) & (~hc[0]);
reg [15:0]vsh;
always @(posedge clk) vsh <= vrd ? d : {vsh[11:0],4'bxxxx};

wire [3:0]pixel;
assign pixel = dblk ? 4'b0000 : vsh[15:12];

// Color palette: CGA-like

assign video[11]=pixel[2];	// R[3]
assign video[10]=pixel[3];	// R[2]
assign video[ 9]=pixel[2];	// R[1]
assign video[ 8]=pixel[3];	// R[0]

assign video[ 7]=pixel[1];	// G[3]
assign video[ 6]=pixel[3];	// G[2]
assign video[ 5]=pixel[1];	// G[1]
assign video[ 4]=pixel[3];	// G[0]

assign video[ 3]=pixel[0];	// B[3]
assign video[ 2]=pixel[3];	// B[2]
assign video[ 1]=pixel[0];	// B[1]
assign video[ 0]=pixel[3];	// B[0]

endmodule


